"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unwrapWETH = exports.wrapETH = exports.isLandDataVerified = exports.isSpaceDataVerified = exports.isAvatarDataVerified = exports.isItemDataVerified = exports.isURIHashVerified = exports.approveERC20 = exports.stripHexPrefix = exports.sha256FromHexString = exports.sha256FromBuffer = exports.chainIdToNetworkName = exports.validateAndParseAddress = exports.validateURI = exports.validateBytes32 = exports.constructBid = exports.constructAsk = exports.validateBidShares = exports.constructBidShares = exports.constructLandData = exports.constructAvatarData = exports.constructSpaceData = exports.constructItemData = void 0;
const tslib_1 = require("tslib");
const address_1 = require("@ethersproject/address");
const tiny_warning_1 = (0, tslib_1.__importDefault)(require("tiny-warning"));
const tiny_invariant_1 = (0, tslib_1.__importDefault)(require("tiny-invariant"));
const sjcl_1 = (0, tslib_1.__importDefault)(require("sjcl"));
const Decimal_1 = require("./Decimal");
const bytes_1 = require("@ethersproject/bytes");
//import { recoverTypedSignature, signTypedData_v4 } from "eth-sig-util";
//import { fromRpcSig, toRpcSig } from "ethereumjs-util";
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const axios_1 = (0, tslib_1.__importDefault)(require("axios"));
const ethers_1 = require("ethers");
/********************
 * Type Constructors
 ********************
 */
function constructItemData(tokenURI, metadataURI, contentHash, metadataHash) {
    // validate the hash to ensure it fits in bytes32
    validateBytes32(contentHash);
    validateBytes32(metadataHash);
    validateURI(tokenURI);
    validateURI(metadataURI);
    return {
        tokenURI: tokenURI,
        metadataURI: metadataURI,
        contentHash: contentHash,
        metadataHash: metadataHash,
    };
}
exports.constructItemData = constructItemData;
function constructSpaceData(tokenURI, metadataURI, contentHash, metadataHash, isPublic, lands) {
    // validate the hash to ensure it fits in bytes32
    validateBytes32(contentHash);
    validateBytes32(metadataHash);
    validateURI(tokenURI);
    validateURI(metadataURI);
    return {
        tokenURI: tokenURI,
        metadataURI: metadataURI,
        contentHash: contentHash,
        metadataHash: metadataHash,
        isPublic: isPublic,
        lands: lands
    };
}
exports.constructSpaceData = constructSpaceData;
function constructAvatarData(tokenURI, metadataURI, contentHash, metadataHash) {
    // validate the hash to ensure it fits in bytes32
    validateBytes32(contentHash);
    validateBytes32(metadataHash);
    validateURI(tokenURI);
    validateURI(metadataURI);
    return {
        tokenURI: tokenURI,
        metadataURI: metadataURI,
        contentHash: contentHash,
        metadataHash: metadataHash
    };
}
exports.constructAvatarData = constructAvatarData;
function constructLandData(tokenURI, metadataURI, contentHash, metadataHash, xCoordinate, yCoordinate) {
    // validate the hash to ensure it fits in bytes32
    validateBytes32(contentHash);
    validateBytes32(metadataHash);
    validateURI(tokenURI);
    validateURI(metadataURI);
    return {
        tokenURI: tokenURI,
        metadataURI: metadataURI,
        contentHash: contentHash,
        metadataHash: metadataHash,
        xCoordinate: xCoordinate,
        yCoordinate: yCoordinate
    };
}
exports.constructLandData = constructLandData;
function constructBidShares(creator, owner, prevOwner) {
    const decimalCreator = Decimal_1.Decimal.new(parseFloat(creator.toFixed(4)));
    const decimalOwner = Decimal_1.Decimal.new(parseFloat(owner.toFixed(4)));
    const decimalPrevOwner = Decimal_1.Decimal.new(parseFloat(prevOwner.toFixed(4)));
    validateBidShares(decimalCreator, decimalOwner, decimalPrevOwner);
    return {
        creator: decimalCreator,
        owner: decimalOwner,
        prevOwner: decimalPrevOwner,
    };
}
exports.constructBidShares = constructBidShares;
function validateBidShares(creator, owner, prevOwner) {
    const decimal100 = Decimal_1.Decimal.new(100);
    const sum = creator.value.add(owner.value).add(prevOwner.value);
    if (sum.toString() != decimal100.value.toString()) {
        (0, tiny_invariant_1.default)(false, `The BidShares sum to ${sum.toString()}, but they must sum to ${decimal100.value.toString()}`);
    }
}
exports.validateBidShares = validateBidShares;
function constructAsk(currency, amount) {
    const parsedCurrency = validateAndParseAddress(currency);
    return {
        currency: parsedCurrency,
        amount: amount,
    };
}
exports.constructAsk = constructAsk;
function constructBid(currency, amount, bidder, recipient, sellOnShare) {
    let parsedCurrency;
    let parsedBidder;
    let parsedRecipient;
    try {
        parsedCurrency = validateAndParseAddress(currency);
    }
    catch (err) {
        throw new Error(`Currency address is invalid: ${err.message}`);
    }
    try {
        parsedBidder = validateAndParseAddress(bidder);
    }
    catch (err) {
        throw new Error(`Bidder address is invalid: ${err.message}`);
    }
    try {
        parsedRecipient = validateAndParseAddress(recipient);
    }
    catch (err) {
        throw new Error(`Recipient address is invalid: ${err.message}`);
    }
    const decimalSellOnShare = Decimal_1.Decimal.new(parseFloat(sellOnShare.toFixed(4)));
    return {
        currency: parsedCurrency,
        amount: amount,
        bidder: parsedBidder,
        recipient: parsedRecipient,
        sellOnShare: decimalSellOnShare,
    };
}
exports.constructBid = constructBid;
function validateBytes32(value) {
    if (typeof value == "string") {
        if ((0, bytes_1.isHexString)(value) && (0, bytes_1.hexDataLength)(value) == 32) {
            return;
        }
        (0, tiny_invariant_1.default)(false, `${value} is not a 0x prefixed 32 bytes hex string`);
    }
    else {
        if ((0, bytes_1.hexDataLength)((0, bytes_1.hexlify)(value)) == 32) {
            return;
        }
        (0, tiny_invariant_1.default)(false, `value is not a length 32 byte array`);
    }
}
exports.validateBytes32 = validateBytes32;
function validateURI(uri) {
    if (!uri.match(/^https:\/\/(.*)/)) {
        (0, tiny_invariant_1.default)(false, `${uri} must begin with \`https://\``);
    }
}
exports.validateURI = validateURI;
function validateAndParseAddress(address) {
    try {
        const checksummedAddress = (0, address_1.getAddress)(address);
        (0, tiny_warning_1.default)(address === checksummedAddress, `${address} is not checksummed.`);
        return checksummedAddress;
    }
    catch (error) {
        (0, tiny_invariant_1.default)(false, `${address} is not a valid address.`);
    }
}
exports.validateAndParseAddress = validateAndParseAddress;
function chainIdToNetworkName(chainId) {
    switch (chainId) {
        case 7018: {
            return "motif";
        }
        case 3: {
            return "ropsten";
        }
        case 1: {
            return "mainnet";
        }
    }
    (0, tiny_invariant_1.default)(false, `chainId ${chainId} not officially supported by the Motif`);
}
exports.chainIdToNetworkName = chainIdToNetworkName;
/********************
 * Hashing Utilities
 ********************
 */
function sha256FromBuffer(buffer) {
    const bitArray = sjcl_1.default.codec.hex.toBits(buffer.toString("hex"));
    const hashArray = sjcl_1.default.hash.sha256.hash(bitArray);
    return "0x".concat(sjcl_1.default.codec.hex.fromBits(hashArray));
}
exports.sha256FromBuffer = sha256FromBuffer;
function sha256FromHexString(data) {
    if (!(0, bytes_1.isHexString)(data)) {
        throw new Error(`${data} is not valid 0x prefixed hex`);
    }
    const bitArray = sjcl_1.default.codec.hex.toBits(data);
    const hashArray = sjcl_1.default.hash.sha256.hash(bitArray);
    return "0x".concat(sjcl_1.default.codec.hex.fromBits(hashArray));
}
exports.sha256FromHexString = sha256FromHexString;
function stripHexPrefix(hex) {
    return hex.slice(0, 2) == "0x" ? hex.slice(2) : hex;
}
exports.stripHexPrefix = stripHexPrefix;
/*********************
 * EIP-712 Utilities
 *********************
 */
/*
export async function signPermitMessage(
  owner: Wallet,
  toAddress: string,
  itemId: number,
  nonce: number,
  deadline: number,
  domain: EIP712Domain
): Promise<EIP712Signature> {
  const tokenId = itemId;

  return new Promise<EIP712Signature>(async (res, reject) => {
    try {
      const sig = signTypedData_v4(
        Buffer.from(owner.privateKey.slice(2), "hex"),
        {
          data: {
            types: {
              EIP712Domain: [
                { name: "name", type: "string" },
                { name: "version", type: "string" },
                { name: "chainId", type: "uint256" },
                { name: "verifyingContract", type: "address" },
              ],
              Permit: [
                { name: "spender", type: "address" },
                { name: "tokenId", type: "uint256" },
                { name: "nonce", type: "uint256" },
                { name: "deadline", type: "uint256" },
              ],
            },
            primaryType: "Permit",
            domain: domain,
            message: {
              spender: toAddress,
              tokenId,
              nonce,
              deadline,
            },
          },
        }
      );

      const response = fromRpcSig(sig);

      res({
        r: response.r,
        s: response.s,
        v: response.v,
        deadline: deadline.toString(),
      });
    } catch (e) {
      console.error(e);
      reject(e);
    }
  });
}

export async function recoverSignatureFromPermit(
  toAddress: string,
  itemId: number,
  nonce: number,
  deadline: number,
  domain: EIP712Domain,
  eipSig: EIP712Signature
) {
  const r = arrayify(eipSig.r);
  const s = arrayify(eipSig.s);

  const tokenId = itemId;

  const recovered = recoverTypedSignature({
    data: {
      types: {
        EIP712Domain: [
          { name: "name", type: "string" },
          { name: "version", type: "string" },
          { name: "chainId", type: "uint256" },
          { name: "verifyingContract", type: "address" },
        ],
        Permit: [
          { name: "spender", type: "address" },
          { name: "tokenId", type: "uint256" },
          { name: "nonce", type: "uint256" },
          { name: "deadline", type: "uint256" },
        ],
      },
      primaryType: "Permit",
      domain: domain,
      message: {
        spender: toAddress,
        tokenId,
        nonce,
        deadline,
      },
    },
    sig: toRpcSig(eipSig.v, Buffer.from(r), Buffer.from(s)),
  });
  return recovered;
}

export async function recoverSignatureFromMintWithSig(
  contentHash: BytesLike,
  metadataHash: BytesLike,
  creatorShareBN: BigNumber,
  nonce: number,
  deadline: number,
  domain: EIP712Domain,
  eipSig: EIP712Signature
) {
  const r = arrayify(eipSig.r);
  const s = arrayify(eipSig.s);
  const creatorShare = creatorShareBN.toString();

  const recovered = recoverTypedSignature({
    data: {
      types: {
        EIP712Domain: [
          { name: "name", type: "string" },
          { name: "version", type: "string" },
          { name: "chainId", type: "uint256" },
          { name: "verifyingContract", type: "address" },
        ],
        MintForCreatorWithSig: [
          { name: "contentHash", type: "bytes32" },
          { name: "metadataHash", type: "bytes32" },
          { name: "creatorShare", type: "uint256" },
          { name: "nonce", type: "uint256" },
          { name: "deadline", type: "uint256" },
        ],
      },
      primaryType: "MintForCreatorWithSig",
      domain: domain,
      message: {
        contentHash,
        metadataHash,
        creatorShare,
        nonce,
        deadline,
      },
    },
    sig: toRpcSig(eipSig.v, Buffer.from(r), Buffer.from(s)),
  });
  return recovered;
}

export async function signMintWithSigMessage(
  owner: Wallet,
  contentHash: BytesLike,
  metadataHash: BytesLike,
  creatorShareBN: BigNumber,
  nonce: number,
  deadline: number,
  domain: EIP712Domain
): Promise<EIP712Signature> {
  try {
    validateBytes32(contentHash);
    validateBytes32(metadataHash);
  } catch (err) {
    return Promise.reject(err.message);
  }

  const creatorShare = creatorShareBN.toString();

  return new Promise<EIP712Signature>(async (res, reject) => {
    try {
      const sig = signTypedData_v4(
        Buffer.from(owner.privateKey.slice(2), "hex"),
        {
          data: {
            types: {
              EIP712Domain: [
                { name: "name", type: "string" },
                { name: "version", type: "string" },
                { name: "chainId", type: "uint256" },
                { name: "verifyingContract", type: "address" },
              ],
              MintForCreatorWithSig: [
                { name: "contentHash", type: "bytes32" },
                { name: "metadataHash", type: "bytes32" },
                { name: "creatorShare", type: "uint256" },
                { name: "nonce", type: "uint256" },
                { name: "deadline", type: "uint256" },
              ],
            },
            primaryType: "MintForCreatorWithSig",
            domain: domain,
            message: {
              contentHash,
              metadataHash,
              creatorShare,
              nonce,
              deadline,
            },
          },
        }
      );
      const response = fromRpcSig(sig);
      res({
        r: response.r,
        s: response.s,
        v: response.v,
        deadline: deadline.toString(),
      });
    } catch (e) {
      console.error(e);
      reject(e);
    }
  });
}*/
function approveERC20(wallet, erc20Address, spender, amount) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const erc20 = typechain_1.BaseErc20Factory.connect(erc20Address, wallet);
        return erc20.approve(spender, amount);
    });
}
exports.approveERC20 = approveERC20;
function isURIHashVerified(uri, expectedHash, timeout = 10) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        try {
            validateURI(uri);
            const resp = yield axios_1.default.get(uri, {
                timeout: timeout,
                responseType: "arraybuffer",
            });
            const uriHash = sha256FromBuffer(resp.data);
            const normalizedExpectedHash = (0, bytes_1.hexlify)(expectedHash);
            return uriHash == normalizedExpectedHash;
        }
        catch (err) {
            return Promise.reject(err.message);
        }
    });
}
exports.isURIHashVerified = isURIHashVerified;
function isItemDataVerified(itemData, timeout = 10) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const isTokenURIVerified = yield isURIHashVerified(itemData.tokenURI, itemData.contentHash, timeout);
        const isMetadataURIVerified = yield isURIHashVerified(itemData.metadataURI, itemData.metadataHash, timeout);
        return isTokenURIVerified && isMetadataURIVerified;
    });
}
exports.isItemDataVerified = isItemDataVerified;
function isAvatarDataVerified(avatarData, timeout = 10) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const isTokenURIVerified = yield isURIHashVerified(avatarData.tokenURI, avatarData.contentHash, timeout);
        const isMetadataURIVerified = yield isURIHashVerified(avatarData.metadataURI, avatarData.metadataHash, timeout);
        return isTokenURIVerified && isMetadataURIVerified;
    });
}
exports.isAvatarDataVerified = isAvatarDataVerified;
function isSpaceDataVerified(spaceData, timeout = 10) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const isTokenURIVerified = yield isURIHashVerified(spaceData.tokenURI, spaceData.contentHash, timeout);
        const isMetadataURIVerified = yield isURIHashVerified(spaceData.metadataURI, spaceData.metadataHash, timeout);
        return isTokenURIVerified && isMetadataURIVerified;
    });
}
exports.isSpaceDataVerified = isSpaceDataVerified;
function isLandDataVerified(landData, timeout = 10) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const isTokenURIVerified = yield isURIHashVerified(landData.tokenURI, landData.contentHash, timeout);
        const isMetadataURIVerified = yield isURIHashVerified(landData.metadataURI, landData.metadataHash, timeout);
        return isTokenURIVerified && isMetadataURIVerified;
    });
}
exports.isLandDataVerified = isLandDataVerified;
function wrapETH(wallet, wethAddress, amount) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const abi = ["function deposit() public payable"];
        const weth = new ethers_1.ethers.Contract(wethAddress, abi, wallet);
        return weth.deposit({ value: amount });
    });
}
exports.wrapETH = wrapETH;
function unwrapWETH(wallet, wethAddress, amount) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const abi = ["function withdraw(uint256) public"];
        const weth = new ethers_1.ethers.Contract(wethAddress, abi, wallet);
        return weth.withdraw(amount);
    });
}
exports.unwrapWETH = unwrapWETH;
//# sourceMappingURL=utils.js.map